"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const case_1 = __importDefault(require("case"));
const utils_1 = require("../../utils");
const name = 'parameter-casing';
function getCasesSetFromOptions(option, defaultCase) {
    return typeof option === 'string' && utils_1.isValidCaseName(option)
        ? utils_1.validCases[option]
        : defaultCase;
}
const PARAMETER_LOCATIONS = [
    'query',
    'header',
    'path',
    'formData',
    'body',
];
const rule = {
    name,
    visitor: {
        ParameterObject: ({ node, report, setting }) => {
            if (typeof setting === 'boolean')
                return;
            const [settingCasingName, opts = {}] = setting;
            if (typeof settingCasingName === 'string' &&
                utils_1.isValidCaseName(settingCasingName)) {
                const defaultParamCase = utils_1.validCases[settingCasingName];
                const cases = {
                    query: getCasesSetFromOptions(opts.query, defaultParamCase),
                    header: getCasesSetFromOptions(opts.header, defaultParamCase),
                    path: getCasesSetFromOptions(opts.path, defaultParamCase),
                    formData: getCasesSetFromOptions(opts.formData, defaultParamCase),
                    body: getCasesSetFromOptions(opts.body, defaultParamCase),
                };
                const IGNORE_PARAMETER_NAMES = new Set(Array.isArray(opts.ignore) ? opts.ignore : []);
                if (IGNORE_PARAMETER_NAMES.has(node.name))
                    return;
                const nodeCase = case_1.default.of(node.name);
                const paramLocation = node.in;
                if (!cases[paramLocation].has(nodeCase)) {
                    const shouldBeCase = cases[paramLocation]
                        .values()
                        .next().value;
                    if (utils_1.isValidCaseName(shouldBeCase) &&
                        !(shouldBeCase in case_1.default)) {
                        return;
                    }
                    const correctVersion = settingCasingName in utils_1.validCases
                        ? case_1.default[shouldBeCase](node.name)
                        : '';
                    report(`Parameter "${node.name}" has wrong casing.${correctVersion
                        ? ` Should be "${correctVersion}".`
                        : ''}`);
                }
            }
        },
    },
    isValidSetting: option => {
        if (typeof option !== 'object')
            return false;
        const [first, second] = option;
        const isValidFirstItem = first in utils_1.validCases;
        if (!isValidFirstItem) {
            return { msg: `"${first}" is not a valid rule setting` };
        }
        if (option.length === 1)
            return true;
        if (utils_1.isObject(second)) {
            if (utils_1.hasKey('ignore', second)) {
                const { ignore } = second;
                if (!Array.isArray(ignore))
                    return {
                        msg: 'Setting contains "ignore" which is not an array.',
                    };
                const isEachIgnoreItemString = ignore.every((x) => typeof x === 'string');
                if (!isEachIgnoreItemString) {
                    return { msg: 'Each item in "ignore" has to be a string.' };
                }
            }
            if (PARAMETER_LOCATIONS.some(name => !utils_1.isValidCaseName(name))) {
                return {
                    msg: 'Settings for parameter location has to be a valid case name.',
                };
            }
            return true;
        }
        else
            return false;
    },
    defaultSetting: ['camel', { header: 'kebab' }],
};
exports.default = rule;
